#!/usr/bin/perl -w
use strict;

use Email::MIME;
use Email::Address;
use Email::MIME::Attachment::Stripper;
use HTML::Strip;
use Getopt::Long;
use Data::Dumper;

my %email = ();
my %emailaddress = ();
my @emails = ();

my ( $pst, $emaildir ) = options();
print "pst=$pst, emaildir=$emaildir\n";
my $targetdatadir = 'wiki/data/Sandbox';
my $targetpubdir  = 'wiki/pub/Sandbox';

my ( $emailfile );
@emails = listfiles( $emaildir );

foreach $emailfile (@emails) {
  next unless ( $emailfile =~ m!\d+\Z! );
#  next unless ( $emailfile =~ m!338\Z! ); ## <<<<<<<<<<<<<< select here

print "$emailfile\n";
  %email = parsemail( $emailfile );

# print Dumper \%email;

  createwikipage( $targetdatadir, $targetpubdir, %email );
}

sub listfiles {
  
  my @filelist = ();
  my $find = "find $_[0] -maxdepth 2 -type f -printf '%h/%f\\n'";
  open LIST, "$find |";
  while( <LIST> ) { 
    chomp;
    push( @filelist, $_ );
  }
  close LIST;
  return @filelist;
}

sub createwikipage {
use List::Util qw(min max);
use File::Copy;
use File::Path;
  my ( $datadir, $pubdir, %email ) = @_;
  my ( $i, $attachment, $attachmentfile, $pagename, $addressee, $tag );
#  print Dumper( \%email );

  $addressee = ( $email{'mbox'} eq 'Inbox' ) ? $email{'from'}[0] : $email{'to'}[0];
  $tag = ( $email{'mbox'} eq 'Inbox' ) ? "InComing" : "OutGoing";
  $pagename = sprintf("%s%s%s",$email{'mbox'}, sortdatetime( $email{'datetime'} ), $addressee);
  $pagename =~ s/[, ();&]//g;  # the ; is an artefact of an poorly quoted email address.
###  print ">>>Page: $pagename\n";

  open PAGE, ">:utf8", "$datadir/$pagename.txt";
  print PAGE "---+ $email{'subject'}\n";
  print PAGE "Date: $email{'datetime'}\n";
  print PAGE "<table><tr><td>\n";
  print PAGE "| *From* | *To* | *CC* |\n";
  for ( $i = 0; $i <= max($#{$email{'from'}},$#{$email{'to'}},$#{$email{'cc'}}); $i++ ) {
    printf PAGE ("|%s|%s|%s|\n",
                     ( defined $email{'from'}[$i] ) ? $email{'from'}[$i] : ' ',
                     ( defined $email{'to'}[$i] )   ? $email{'to'}[$i] : ' ',
                     ( defined $email{'cc'}[$i] )   ? $email{'cc'}[$i] : ' '   );

  }
  print PAGE "</td><td>\n";
  foreach $attachment ( @{$email{'attachments'}} ) {
    mkpath ( "$pubdir/$pagename" );
    ( $attachmentfile ) = ( $attachment =~ m!/([^/]+)\Z! );
#    print "cp $pstdir/$email{'mbox'}/$attachment $pubdir/$pagename/$attachment\n";
    copy ( "$attachment", "$pubdir/$pagename/$attachmentfile" );
    print PAGE "[[%ATTACHURL%/$attachmentfile][$attachmentfile]]<br />\n";
  }
  print PAGE "</td></tr></table>\n"; 
#if ( !defined $email{'textbody'} ) {print "Undefined textbody line 48:\n"; print &Dumper(\%email)};
  print PAGE $email{'body'};

# Now create the meta data

print PAGE "\n\n";
  print PAGE '%META:TOPICPARENT{name="WebHome"}%' . "\n";
  print PAGE '%META:FORM{name="Applications/MailArchiveApp.MailArchiveForm"}%' . "\n";
  print PAGE '%META:FIELD{name="TopicType" title="TopicType" value="MailArchiveTopic, WikiTopic"}%' . "\n";
  print PAGE '%META:FIELD{name="TopicTitle" attributes="" title="<nop>TopicTitle" value="'
           . $email{'subject'} . "\"}%\n";
  print PAGE '%META:FIELD{name="Summary" attributes="" title="Summary" value="' . $email{'datetime'} . "\"}%\n";
  print PAGE '%META:FIELD{name="Mailbox" attributes="" title="Mailbox" value="' . $tag . "\"}%\n";
print PAGE q[%META:FIELD{name="To" attributes="" title="To" value="]
           . join( ", ", @{$email{'to'}} ) . qq["}%\n];
print PAGE q[%META:FIELD{name="From" attributes="" title="From" value="]
           . join( ", ", @{$email{'from'}} ) . qq["}%\n];
print PAGE q[%META:FIELD{name="CC" attributes="" title="CC" value="]
           . join( ", ", @{$email{'cc'}} )  . qq["}%\n];
print PAGE q[%META:FIELD{name="Date" attributes="" title="Date" value="]
           . $email{'datetime'} . qq["}%\n];
my $s = $email{'subject'};
   $s =~ s!\A(Re|Fw|:|\s)*!!gi;
print PAGE q[%META:FIELD{name="Subject" attributes="" title="Subject" value="]
           . $s . qq["}%\n];

  close PAGE;
}

sub sortdatetime {
  my ($datetime ) = @_;
  
  use Date::Parse;
 
  my ($ss,$mm,$hh,$day,$month,$year,$zone) = strptime($datetime);
  return "NoDate" unless defined $year;
  return sprintf("%4d%02d%02d%02d%02d%02d",$year+1900, $month+1,$day,$hh,$mm,$ss)
}

sub formatname {
  my ($name ) = @_;
  if ( $name =~ m/\A([^,]*),(.*)\Z/ ) { $name = $2 . " " . $1 }
  $name =~ s/\A[\s'"]+|[\s'"]+\Z//g;  # trim spaces and quotes from start and end.
  return $name;
}

sub saveemail {
  my ( $source, $name, $address ) = @_;
  
  push( @{$email{$source}}, $name );
  $emailaddress{$address} = $name unless exists $emailaddress{$address};
}

sub parsemail {
  print ">>>>> parsemail: $_[0]\n";
  %email = pstemaildata( $_[0] );

  my $m = Email::MIME->new( slurp($_[0]) );

## print Dumper $m;
  
  for (Email::Address->parse($m->header('To')))   { saveemail('to',   formatname($_->name), $_->address ) } 
  for (Email::Address->parse($m->header('From'))) { saveemail('from', formatname($_->name), $_->address ) } 
  for (Email::Address->parse($m->header('CC')))   { saveemail('cc',   formatname($_->name), $_->address ) } 
  $email{'datetime'} = $m->header('Date'); 
  $email{'subject'} =  $m->header('Subject'); 

## print Dumper \%email;

  $email{'body'}    =  parsebody( $m );

  return %email;
}

sub pstemaildata {
  my ( $filename ) = @_;
  my %email = ();
# /tmp/email/2015backup/Inbox/
  my ( $year, $mbox, $fileno ) = ( $filename =~ m!/(\d+)\w*/(\w+)/(\d+)\Z! );

  $email{'year'} = $year;
  $email{'mbox'} = $mbox;
  $email{'fileno'} = $fileno;

  push @{$email{'attachments'}}, grep( /$filename-/, @emails);

  return %email;
}

sub formattextbody {
##  my ( $body ) = @_;
  my $body = $_[0]->body();

  $body =~ s!\n[\n\s]*\n!%PARA%!gs;
  $body =~ s!\n!<br />\n!gs;
  $body =~ s!%PARA%!\n\n!gs;

  return $body;
}

sub formathtmlbody {
##  my ( $body ) = @_;
  my $body = $_[0]->body();
  my $hs = HTML::Strip->new();

  $body =~ s!<br[^>]*>!%BREAK%!gs;
  $body =~ s!</?a([^>]*)>!%HREF{$1}%!gs;
  $body = $hs->parse( $body );
  $hs->eof();
  $body =~ s!%HREF{}%!]]!gs;                                # create href wiki syntax
  $body =~ s!%HREF{.*?href=["']([^"']+)["'].*?}%![[$1][!gs; # get the url
  $body =~ s!\[([^\]]*?)%BREAK%([^\]]*)\]![$1$2]!gs;        # remove %BREAK% from href
  $body =~ s!\n\s*%BREAK%\s*\n!!gs;                         # remove %BREAK alone on a line
  $body =~ s!%BREAK%%BREAK%!\n\n!gs;                        # create paragraphs 
  $body =~ s!(%BREAK%)?\n[\n\s]*\n!\n\n!gs;                 # remove breaks before paragraphs
  $body =~ s!%BREAK%!<br />!gs;                             # insert line breaks.

   return $body;

}

sub formatmixedbody {
##  my ( @parts ) = @_;
  my @parts = $_[0]->parts();
  my ( $p, $body );
###  print "formatmixedbody - parts count: " . $#parts . "\n";
  $body = '';

  if ( $_[0] == $parts[0] ) { print "Empty mixed body - exit\n"; return $body; }

  foreach $p ( @parts ) { 
###    print "FMB::Content-Type: ", $p->content_type('type'), "\n";
#  print Dumper $p->body_raw();
    $body = $body . "\n---\n" . parsebody( $p )
  }
  return $body;
}

sub formatalternativebody {
#  my ( @parts ) = @_;
  my @parts = $_[0]->parts();
  my ( $p, $body, $i, $ct );
###  print "FAB::formatalternativebody - parts count: " . $#parts . "\n";
  $ct = -1;

  if ( $_[0] == $parts[0] ) { print "Empty aalternative body - exit\n"; return $body; }

  for( $i = 0; $i <= $#parts; $i++ ) {
    $ct = $i if ( ($ct == -1) && ($parts[$i]->content_type('type') =~ m!text/html!) );
    $ct = $i if ( $parts[$i]->content_type('type') =~ m!text/plain! );
  }
  if ( $ct == -1 ) { print "CT = $ct - Unhandled ALTERNATIVE content type: ", Dumper \@parts }
  else { $body = parsebody( $parts[$ct] ) }
  

return $body;
}

sub parsebody {
  my ( $m ) = @_;
  my ( $body, $p );
#print ">>>>> parsebody: \n";
  $body = '';

  if ( $m->content_type('type') =~ m!multipart/mixed! ) {
#Note types are: multipart/alternative, multipart/mixed
###    print "multipart/mixed: parts count: " . $m->parts . "\n";
    $body = formatmixedbody( $m );
  } elsif ( $m->content_type('type') =~ m!multipart/alternative! ) {
###    print "multipart/alternative: parts count: " . $m->parts . "\n";
    $body = formatalternativebody( $m );
  } elsif ( $m->content_type('type') =~ m!text/plain! ) {
      $body = formattextbody( $m );
  } elsif ( $m->content_type('type') =~ m!text/html! ) {
      $body = formathtmlbody( $m );
  } elsif ( $m->content_type('type') =~ m!\A\s*\Z! ) {
      ### DO NOTHING
  } else {  
    print "Unhandled content-type: " . $m->content_type('type') . "\n";
#Note: there are a lot of application/... content type.
###    print Dumper $m;
    
  }
  return $body;

#abandoned attachment handling
  my $a = Email::MIME::Attachment::Stripper->new($_[0]);
  print "Attachment count: $#{$a->attachments}\n" if defined $a->attachments;;

  print Dumper $a;


  return $a->message;
}

sub options {
my $pst = ''; #set to false
my $emaildir = '/tmp/email/2015backup/Inbox'; ##/tmp/email/
  GetOptions ('pst' => \$pst, 'emaildir=s' => \$emaildir);
  return ( $pst, $emaildir );
}
 
sub slurp {
  my( $filename ) = @_;
  local $/;
  open FILE, "<$filename"; my $file = <FILE>; close FILE;
  return $file;
}

__END__
